package com.hw.ems.base.service;

import com.hw.common.core.domain.Ztree;
import com.hw.ems.base.domain.BaseLineMonitor;

import java.util.List;


/**
 * 线路计量信息Service接口
 *
 * @author YinQ
 * @date 2023-04-22
 */
public interface IBaseLineMonitorService
{
    /**
     * 查询线路计量信息
     *
     * @param objid 线路计量信息主键
     * @return 线路计量信息
     */
    public BaseLineMonitor selectBaseLineMonitorByObjid(Long objid);

    /**
     * 查询线路计量信息列表
     *
     * @param baseLineMonitor 线路计量信息
     * @return 线路计量信息集合
     */
    public List<BaseLineMonitor> selectBaseLineMonitorList(BaseLineMonitor baseLineMonitor);

    /**
     * 新增线路计量信息
     *
     * @param baseLineMonitor 线路计量信息
     * @return 结果
     */
    public int insertBaseLineMonitor(BaseLineMonitor baseLineMonitor);

    /**
     * 修改线路计量信息
     *
     * @param baseLineMonitor 线路计量信息
     * @return 结果
     */
    public int updateBaseLineMonitor(BaseLineMonitor baseLineMonitor);

    /**
     * 批量删除线路计量信息
     *
     * @param objids 需要删除的线路计量信息主键集合
     * @return 结果
     */
    public int deleteBaseLineMonitorByObjids(Long[] objid);

    /**
     * 删除线路计量信息信息
     *
     * @param objid 线路计量信息主键
     * @return 结果
     */
    public int deleteBaseLineMonitorByObjid(Long objid);

    /**
     * 查询线路计量信息树列表
     *
     * @return 所有线路计量信息信息
     */
    public List<Ztree> selectBaseLineMonitorTree(BaseLineMonitor lineMonitor);

    /**
     * 查询计量设备有子集的父级List
     * @param baseLineMonitor 计量设备信息
     * @return
     */
    public List<BaseLineMonitor> selectMonitorSubsetList(BaseLineMonitor baseLineMonitor);
}
