package com.hw.ems.base.service;


import com.hw.ems.base.domain.BaseMonitorInfo;
import com.hw.ems.base.domain.BaseTree;
import com.hw.ems.base.domain.BaseTypeRelation;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

/**
 * 计量设备信息Service接口
 *
 * @author YinQ
 * @date 2023-04-06
 */
public interface IBaseMonitorInfoService {
    /**
     * 查询计量设备信息
     *
     * @param objid 计量设备信息主键
     * @return 计量设备信息
     */
    public BaseMonitorInfo selectBaseMonitorInfoByObjid(Long objid);

    /**
     * 查询计量设备信息列表
     *
     * @param baseMonitorInfo 计量设备信息
     * @return 计量设备信息集合
     */
    public List<BaseMonitorInfo> selectBaseMonitorInfoList(BaseMonitorInfo baseMonitorInfo);

    /**
     * 新增计量设备信息
     *
     * @param baseMonitorInfo 计量设备信息
     * @return 结果
     */
    public int insertBaseMonitorInfo(BaseMonitorInfo baseMonitorInfo);

    /**
     * 修改计量设备信息
     *
     * @param baseMonitorInfo 计量设备信息
     * @return 结果
     */
    public int updateBaseMonitorInfo(BaseMonitorInfo baseMonitorInfo);

    List<BaseMonitorInfo> selectChildrenMonitorById(String monitorId);

    /**
     * 批量删除计量设备信息
     *
     * @param objids 需要删除的计量设备信息主键集合
     * @return 结果
     */
    public int deleteBaseMonitorInfoByObjids(String objids);

    /**
     * 删除计量设备信息信息
     *
     * @param objid 计量设备信息主键
     * @return 结果
     */
    public int deleteBaseMonitorInfoByObjid(Long objid);

    /**
     * 查询计量设备信息树列表
     *
     * @return 所有计量设备信息信息
     */
    public List<BaseTree> selectBaseMonitorInfoTree(Long monitorType);

    /**
     * monitor获取所有子集
     *
     * @param monitorInfoList
     * @return
     */
    public HashMap<String, List<BaseMonitorInfo>> buildMonitorMap(List<BaseMonitorInfo> monitorInfoList);

    /**
     * 查询计量设备有子集的父级
     *
     * @param baseMonitorInfo 计量设备信息
     * @return 计量设备信息集合
     */
    public List<BaseMonitorInfo> selectMonitorSubsetList(BaseMonitorInfo baseMonitorInfo);

    /**
     * 查询类型园区下第一级
     * @param type
     * @return
     */
    ArrayList<BaseTypeRelation> selectTypeSubset(Long type);
    List<BaseMonitorInfo> monitorLedgerList(BaseMonitorInfo baseMonitorInfo);

    List<BaseMonitorInfo> selectMonitor(Long publicShareId);

    List<BaseMonitorInfo> selectDistinctPublicMonitor(String[] ids);

    BigDecimal queryPublicMonitorExpend(String[] ids, String startTime, String endTime, Long energyType);
}
