package com.hw.ems.base.service.impl;

import java.util.List;

import com.hw.common.core.utils.DateUtils;
import com.hw.ems.base.service.IBaseCollectDeviceInfoService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.hw.ems.base.mapper.BaseCollectDeviceInfoMapper;
import com.hw.ems.base.domain.BaseCollectDeviceInfo;

/**
 * 采集设备信息Service业务层处理
 *
 * @author sf
 * @date 2023-04-20
 */
@Service
//@DS("#header.poolName")
public class BaseCollectDeviceInfoServiceImpl implements IBaseCollectDeviceInfoService {
    @Autowired
    private BaseCollectDeviceInfoMapper baseCollectDeviceInfoMapper;

    /**
     * 查询采集设备信息
     *
     * @param objid 采集设备信息主键
     * @return 采集设备信息
     */
    @Override
    //@DS("#header.poolName")
    public BaseCollectDeviceInfo selectBaseCollectDeviceInfoByObjid(Long objid) {
        return baseCollectDeviceInfoMapper.selectBaseCollectDeviceInfoByObjid(objid);
    }

    /**
     * 查询采集设备信息列表
     *
     * @param baseCollectDeviceInfo 采集设备信息
     * @return 采集设备信息
     */
    @Override
    //@DS("#header.poolName")
    public List<BaseCollectDeviceInfo> selectBaseCollectDeviceInfoList(BaseCollectDeviceInfo baseCollectDeviceInfo) {
        return baseCollectDeviceInfoMapper.selectBaseCollectDeviceInfoList(baseCollectDeviceInfo);
    }

    /**
     * 新增采集设备信息
     *
     * @param baseCollectDeviceInfo 采集设备信息
     * @return 结果
     */
    @Override
    //@DS("#header.poolName")
    public int insertBaseCollectDeviceInfo(BaseCollectDeviceInfo baseCollectDeviceInfo) {
        baseCollectDeviceInfo.setCreateTime(DateUtils.getNowDate());
        return baseCollectDeviceInfoMapper.insertBaseCollectDeviceInfo(baseCollectDeviceInfo);
    }

    /**
     * 修改采集设备信息
     *
     * @param baseCollectDeviceInfo 采集设备信息
     * @return 结果
     */
    @Override
    //@DS("#header.poolName")
    public int updateBaseCollectDeviceInfo(BaseCollectDeviceInfo baseCollectDeviceInfo) {
        baseCollectDeviceInfo.setUpdateTime(DateUtils.getNowDate());
        return baseCollectDeviceInfoMapper.updateBaseCollectDeviceInfo(baseCollectDeviceInfo);
    }

    /**
     * 批量删除采集设备信息
     *
     * @param objids 需要删除的采集设备信息主键
     * @return 结果
     */
    @Override
    //@DS("#header.poolName")
    public int deleteBaseCollectDeviceInfoByObjids(Long[] objids) {
        return baseCollectDeviceInfoMapper.deleteBaseCollectDeviceInfoByObjids(objids);
    }

    /**
     * 删除采集设备信息信息
     *
     * @param objid 采集设备信息主键
     * @return 结果
     */
    @Override
    //@DS("#header.poolName")
    public int deleteBaseCollectDeviceInfoByObjid(Long objid) {
        return baseCollectDeviceInfoMapper.deleteBaseCollectDeviceInfoByObjid(objid);
    }
}
