package com.hw.ems.base.service.impl;

import java.util.List;

import com.hw.common.core.utils.DateUtils;
import com.hw.ems.base.mapper.BaseEnergyTypeMapper;
import com.hw.ems.base.service.IBaseEnergyTypeService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.hw.ems.base.domain.BaseEnergyType;

/**
 * 能源类型信息Service业务层处理
 *
 * @author sf
 * @date 2023-04-20
 */
@Service
public class BaseEnergyTypeServiceImpl implements IBaseEnergyTypeService {
    @Autowired
    private BaseEnergyTypeMapper baseEnergyTypeMapper;

    /**
     * 查询能源类型信息
     *
     * @param objid 能源类型信息主键
     * @return 能源类型信息
     */
    @Override
    //@DS("#header.poolName")
    public BaseEnergyType selectBaseEnergyTypeByObjid(Long objid) {
        return baseEnergyTypeMapper.selectBaseEnergyTypeByObjid(objid);
    }

    /**
     * 查询能源类型信息列表
     *
     * @param baseEnergyType 能源类型信息
     * @return 能源类型信息
     */
    @Override
    //@DS("#header.poolName")
    public List<BaseEnergyType> selectBaseEnergyTypeList(BaseEnergyType baseEnergyType) {
//        System.out.println("++++++++++++++++" + baseEnergyType.getPoolName());
        return baseEnergyTypeMapper.selectBaseEnergyTypeList(baseEnergyType);
    }

    /**
     * 新增能源类型信息
     *
     * @param baseEnergyType 能源类型信息
     * @return 结果
     */
    @Override
    //@DS("#header.poolName")
    public int insertBaseEnergyType(BaseEnergyType baseEnergyType) {
        baseEnergyType.setCreateTime(DateUtils.getNowDate());
        return baseEnergyTypeMapper.insertBaseEnergyType(baseEnergyType);
    }

    /**
     * 修改能源类型信息
     *
     * @param baseEnergyType 能源类型信息
     * @return 结果
     */
    @Override
    //@DS("#header.poolName")
    public int updateBaseEnergyType(BaseEnergyType baseEnergyType) {
        baseEnergyType.setUpdateTime(DateUtils.getNowDate());
        return baseEnergyTypeMapper.updateBaseEnergyType(baseEnergyType);
    }

    /**
     * 批量删除能源类型信息
     *
     * @param objids 需要删除的能源类型信息主键
     * @return 结果
     */
    @Override
    //@DS("#header.poolName")
    public int deleteBaseEnergyTypeByObjids(Long[] objids) {
        return baseEnergyTypeMapper.deleteBaseEnergyTypeByObjids(objids);
    }

    /**
     * 删除能源类型信息信息
     *
     * @param objid 能源类型信息主键
     * @return 结果
     */
    @Override
    //@DS("#header.poolName")
    public int deleteBaseEnergyTypeByObjid(Long objid) {
        return baseEnergyTypeMapper.deleteBaseEnergyTypeByObjid(objid);
    }
}
