package com.hw.ems.record.service.impl;

import java.math.BigDecimal;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.*;


import com.hw.common.core.constant.SecurityConstants;
import com.hw.common.core.constant.TdEngineConstants;
import com.hw.common.core.domain.R;
import com.hw.common.core.text.Convert;
import com.hw.common.core.utils.DateUtils;
import com.hw.common.core.utils.StringUtils;
import com.hw.ems.base.domain.BaseEnergyType;
import com.hw.ems.base.domain.BaseMonitorInfo;
import com.hw.ems.base.service.IBaseEnergyTypeService;
import com.hw.ems.base.service.IBaseMonitorInfoService;
import com.hw.ems.record.domain.RecordDnbInstant;
import com.hw.ems.record.mapper.RecordDnbInstantMapper;
import com.hw.ems.record.service.IRecordDnbInstantService;
import com.hw.ems.report.domain.ReportPointDnb;
import com.hw.ems.report.service.IReportPointDnbService;
import com.hw.tdengine.api.RemoteTdEngineService;
import com.hw.tdengine.api.domain.TdHistorySelectDto;
import com.hw.tdengine.api.domain.TdReturnDataVo;
import com.hw.tdengine.api.domain.TdSuperTableSelectVo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

/**
 * 电实时数据Service业务层处理
 *
 * @author YinQ
 * @date 2023-04-07
 */
@Service
public class RecordDnbInstantServiceImpl implements IRecordDnbInstantService {
    @Autowired
    private RecordDnbInstantMapper recordDnbInstantMapper;

    @Autowired
    private IBaseMonitorInfoService baseMonitorInfoService;

    @Autowired
    private RemoteTdEngineService remoteTdEngineService;

    @Autowired
    private IBaseEnergyTypeService baseEnergyTypeService;
    @Autowired
    private IReportPointDnbService reportPointDnbService;


    /**
     * 查询电实时数据
     *
     * @param objid 电实时数据主键
     * @return 电实时数据
     */
    @Override
    //@DS("#header.poolName")
    public RecordDnbInstant selectRecordDnbInstantByObjid(Long objid) {
//        return recordDnbInstantMapper.selectRecordDnbInstantByObjid(objid);
        return new RecordDnbInstant();
    }

    /**
     * 查询电实时数据列表
     *
     * @param recordDnbInstant 电实时数据
     * @return 电实时数据
     */
//    @Override
//    @DS("#header.poolName")
////    @DataScope(deptAlias = "null")
//    public List<RecordDnbInstant> selectRecordDnbInstantList(RecordDnbInstant recordDnbInstant) {
//        return recordDnbInstantMapper.selectRecordDnbInstantList(recordDnbInstant);
//    }
    @Override
    //@DS("#header.poolName")
//    @DataScope(deptAlias = "null")
    public  List<RecordDnbInstant> selectRecordDnbInstantList(RecordDnbInstant recordDnbInstant) {
//   todo 调用tde引擎查询实时数据
        try {
            List<Map<String,Object>> list = new ArrayList<Map<String,Object>>();
            List<RecordDnbInstant> records = new ArrayList<RecordDnbInstant>();
            List<BaseMonitorInfo> baseMonitorInfos = baseMonitorInfoService.selectBaseMonitorInfoList(new BaseMonitorInfo());
            String stratTime = (String)recordDnbInstant.getParams().get("beginCollectTime");
            String endTime = (String)recordDnbInstant.getParams().get("endCollectTime");
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
            Date dateStrat = sdf.parse(stratTime);
            Date dateEnd = sdf.parse(endTime);
            TdHistorySelectDto tdHistorySelectDto = new TdHistorySelectDto();
            tdHistorySelectDto.setDatabaseName("db_hwmes");
            /**
             * 先判断是要查询总的电实时数据还是查询单个计量设备的电实时数据
             * */
//            if (recordDnbInstant.getMonitorId()==""||recordDnbInstant.getMonitorId()==null) {
//                TdSuperTableSelectVo tdSuperTableSelectVo = new TdSuperTableSelectVo();
//                tdSuperTableSelectVo.setDatabaseName("db_hwmes");
//                tdSuperTableSelectVo.setSuperTableName("st_electricity");
//                //获取13位的时间戳
//                tdSuperTableSelectVo.setStartTime(dateStrat.getTime());
//                tdSuperTableSelectVo.setEndTime(dateEnd.getTime());
////                R<List<Map<String, Object>>> latestDataByTags = remoteTdEngineService.getLatestDataByTags(tdSuperTableSelectVo, SecurityConstants.INNER);
////                List<Map<String, Object>> data = latestDataByTags.getData();
//                return remoteTdEngineService.getLatestDataByTags(tdSuperTableSelectVo, SecurityConstants.INNER).getData();
//            }
            for (BaseMonitorInfo baseMonitorInfo:baseMonitorInfos){
                tdHistorySelectDto.setTableName(TdEngineConstants.getEmsTableName(baseMonitorInfo.getMonitorId()));
                tdHistorySelectDto.setStartTime(dateStrat.getTime());
                tdHistorySelectDto.setEndTime(dateEnd.getTime());
                tdHistorySelectDto.setFirstFieldName("ts");
//                list = (List<Map<String,Object>>)remoteTdEngineService.getHistoryData(tdHistorySelectDto, SecurityConstants.INNER);
//                list.addAll((List<Map<String,Object>>)remoteTdEngineService.getHistoryData(tdHistorySelectDto, SecurityConstants.INNER));
                List<Map<String, Object>> dataList = remoteTdEngineService.getHistoryData(tdHistorySelectDto, SecurityConstants.INNER).getData().getDataList();
                for (Map<String, Object> data : dataList){
                    RecordDnbInstant recordDnbInstant1 = new RecordDnbInstant();
                    recordDnbInstant1.setObjid(baseMonitorInfo.getObjid());
                    recordDnbInstant1.setMonitorId(baseMonitorInfo.getMonitorId());
                    recordDnbInstant1.setMonitorName(baseMonitorInfo.getMonitorName());
                    recordDnbInstant1.setVA(BigDecimal.valueOf(Double.valueOf(String.valueOf(data.get("volatagea")))));
                    recordDnbInstant1.setVB(BigDecimal.valueOf(Double.valueOf(String.valueOf(data.get("volatageb")))));
                    recordDnbInstant1.setVC(BigDecimal.valueOf(Double.valueOf(String.valueOf(data.get("volatagec")))));
                    recordDnbInstant1.setIA(BigDecimal.valueOf(Double.valueOf(String.valueOf(data.get("currenta")))));
                    recordDnbInstant1.setIB(BigDecimal.valueOf(Double.valueOf(String.valueOf(data.get("currentb")))));
                    recordDnbInstant1.setIC(BigDecimal.valueOf(Double.valueOf(String.valueOf(data.get("currentc")))));
                    recordDnbInstant1.setGlys(BigDecimal.valueOf(Double.valueOf(String.valueOf(data.get("glys")))));
                    recordDnbInstant1.setZxyg(BigDecimal.valueOf(Double.valueOf(String.valueOf(data.get("zxyg")))));
                    recordDnbInstant1.setActivePower(BigDecimal.valueOf(Double.valueOf(String.valueOf(data.get("activepower")))));
                    recordDnbInstant1.setReactivePower(BigDecimal.valueOf(Double.valueOf(String.valueOf(data.get("reactivepower")))));
                    //调整时间格式
                    String substring = String.valueOf(data.get("ts")).replaceAll("T", " ").substring(0, 19);
                    recordDnbInstant1.setCollectTime(sdf.parse(substring));
//            recordDnbInstant.setConsumption(BigDecimal.valueOf(Double.valueOf(String.valueOf(data.get(0).get("consuption")))));
                    recordDnbInstant1.setCollectType(Long.valueOf(String.valueOf(data.get("collecttype"))));
                    records.add(recordDnbInstant1);
                }
            }
            return records;
        } catch (ParseException e) {
            e.printStackTrace();
        }
        List<RecordDnbInstant> lists = new ArrayList<>();
            return lists;
    }

    /**
     * 新增电实时数据
     *
     * @param recordDnbInstant 电实时数据
     * @return 结果
     */
    @Override
    //@DS("#header.poolName")
    public int insertRecordDnbInstant(RecordDnbInstant recordDnbInstant) {
        /*if (StringUtils.isNotEmpty(recordDnbInstant.getMonitorId())){
            RecordDnbInstant dnbInstant = new RecordDnbInstant();
            dnbInstant.setMonitorId(recordDnbInstant.getMonitorId());
            List<RecordDnbInstant> recordDnbInstants = recordDnbInstantMapper.instantList(dnbInstant);
            if (recordDnbInstants.size() > 0){
                for (RecordDnbInstant instant : recordDnbInstants) {
                    if (instant.getZxyg().compareTo(recordDnbInstant.getZxyg()) > 0){
                        return 0;
                    }
                }
            }
        }*/
        recordDnbInstant.setRecordTime(new Date());
        recordDnbInstant.setCollectType(0L);
        return recordDnbInstantMapper.insertRecordDnbInstant(recordDnbInstant);
    }

    /**
     * 修改电实时数据
     *
     * @param recordDnbInstant 电实时数据
     * @return 结果
     */
    @Override
    //@DS("#header.poolName")
    public int updateRecordDnbInstant(RecordDnbInstant recordDnbInstant) {
        recordDnbInstant.setRecordTime(new Date());
        return recordDnbInstantMapper.updateRecordDnbInstant(recordDnbInstant);
    }

    /**
     * 批量删除电实时数据
     *
     * @param objids 需要删除的电实时数据主键
     * @return 结果
     */
    @Override
    //@DS("#header.poolName")
    public int deleteRecordDnbInstantByObjids(String objids) {
        return recordDnbInstantMapper.deleteRecordDnbInstantByObjids(Convert.toStrArray(objids));
    }

    /**
     * 删除电实时数据信息
     *
     * @param objid 电实时数据主键
     * @return 结果
     */
    @Override
    //@DS("#header.poolName")
    public int deleteRecordDnbInstantByObjid(Long objid) {
        return recordDnbInstantMapper.deleteRecordDnbInstantByObjid(objid);
    }

    /**
     * 实时数据
     *
     * @param recordDnbInstant
     * @return
     */
    @Override
    //@DS("#header.poolName")
    public List<RecordDnbInstant> instantList(RecordDnbInstant recordDnbInstant) {
        //是空 直接添加
        if (StringUtils.isEmpty(recordDnbInstant.getMonitorSubset())) {
            return recordDnbInstantMapper.instantList(new RecordDnbInstant());
        }
        List<RecordDnbInstant> dnbInstantList = new ArrayList<>();
        BaseMonitorInfo monitorInfo = new BaseMonitorInfo();
        //查询子集加入到父id中
        monitorInfo.setParentId(Long.valueOf(recordDnbInstant.getMonitorSubset()));
        monitorInfo.setParams(null);
        recordDnbInstant.setParams(null);
        //查询id为getMonitorSubset的子集
        List<BaseMonitorInfo> baseMonitorInfos = baseMonitorInfoService.selectBaseMonitorInfoList(monitorInfo);
        for (BaseMonitorInfo baseMonitorInfo : baseMonitorInfos) {
            RecordDnbInstant dnbInstant = new RecordDnbInstant();
            dnbInstant.setMonitorId(baseMonitorInfo.getMonitorId());
            dnbInstant.setMonitorName(baseMonitorInfo.getMonitorName());
            RecordDnbInstant baseDnbInstant = new RecordDnbInstant();
            baseDnbInstant.setMonitorSubset(baseMonitorInfo.getMonitorId());
            List<RecordDnbInstant> dnbInstantListOld = recordDnbInstantMapper.instantList(baseDnbInstant);
            if (dnbInstantListOld.size() > 0) {
                BigDecimal zxyg = dnbInstantListOld.stream().map(RecordDnbInstant::getZxyg).reduce(BigDecimal.ZERO, BigDecimal::add);
                Date collectTime = dnbInstantListOld.stream().map(RecordDnbInstant::getCollectTime).max(Date::compareTo).orElse(null);
                dnbInstant.setZxyg(zxyg);
                dnbInstant.setCollectTime(collectTime);
            }
            dnbInstantList.add(dnbInstant);
        }
        return dnbInstantList;
    }

    /**
     * 获取最新实时电数据
     *  需要先排序在查询出每个设备的最新数据
     * @param baseDnbInstant
     * @return
     */
    @Override
    //@DS("#header.poolName")
    public List<RecordDnbInstant> selectRealTimeParentList(RecordDnbInstant baseDnbInstant) {
        List<RecordDnbInstant> dnbInstantList = new ArrayList<>();
        if (StringUtils.isEmpty(baseDnbInstant.getMonitorSubset())) {
            return recordDnbInstantMapper.instantList(new RecordDnbInstant());
        }
        RecordDnbInstant dnbInstant = new RecordDnbInstant();
        BaseMonitorInfo monitorInfo = new BaseMonitorInfo();
        monitorInfo.setMonitorId(baseDnbInstant.getMonitorSubset());
        baseDnbInstant.setParams(null);
        List<BaseMonitorInfo> baseMonitorInfos = baseMonitorInfoService.selectBaseMonitorInfoList(monitorInfo);
        if (baseMonitorInfos.size() > 0) {

            List<RecordDnbInstant> dnbInstantListOld = recordDnbInstantMapper.instantList(baseDnbInstant);
            dnbInstant.setMonitorId(baseMonitorInfos.get(0).getMonitorId());
            dnbInstant.setMonitorName(baseMonitorInfos.get(0).getMonitorName());
            if (dnbInstantListOld.size() > 0) {
                BigDecimal zxyg = dnbInstantListOld.stream().map(RecordDnbInstant::getZxyg).reduce(BigDecimal.ZERO, BigDecimal::add);
                Date collectTime = dnbInstantListOld.stream().map(RecordDnbInstant::getCollectTime).max(Date::compareTo).orElse(null);
                dnbInstant.setZxyg(zxyg);
                dnbInstant.setCollectTime(collectTime);
            }
        }
//        HashMap<String, List<BaseMonitorInfo>> childrenMonitorInfoMap = baseMonitorInfoService.buildMonitorMap(baseMonitorInfos);
//        List<BaseMonitorInfo> baseMonitorList = childrenMonitorInfoMap.get(baseDnbInstant.getMonitorSubset());
//        List<String> monitorList  = baseMonitorList.stream().map(BaseMonitorInfo::getMonitorId).collect(Collectors.toList());
//        HashMap<String, Object> paramMap = new HashMap<>();
//        paramMap.put("monitorList", monitorList);
//        baseDnbInstant.setParams(paramMap);
//
//        List<BaseMonitorInfo> collect = baseMonitorInfos.stream().filter(e -> e.getMonitorId().equals(baseDnbInstant.getMonitorSubset())).collect(Collectors.toList());
//
        dnbInstantList.add(dnbInstant);
        return dnbInstantList;
    }


    /**
     * 表具实时状态
     *
     * @param dnbInstant
     * @return
     */
    @Override
    //@DS("#header.poolName")
    public RecordDnbInstant tablewareDetailsInfo(RecordDnbInstant dnbInstant) {
        if (StringUtils.isEmpty(dnbInstant.getMonitorId())) {
            return null;
        }
        List<RecordDnbInstant> recordDnbInstants = recordDnbInstantMapper.instantList(dnbInstant);
        if (recordDnbInstants.size() == 0) {
            return null;
        }
        return recordDnbInstants.get(0);
    }

    @Override
    //@DS("#header.poolName")
    public List<RecordDnbInstant> voltageCurrentCurve(RecordDnbInstant dnbInstant) {
        if (StringUtils.isEmpty(dnbInstant.getMonitorId())){
            return null;
        }
        if (StringUtils.isEmpty(dnbInstant.getStartTime()) || StringUtils.isEmpty(dnbInstant.getEndTime())){
            String nowTime = DateUtils.getTime();
            DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
            LocalDateTime modifiedDateTime = LocalDateTime.parse(nowTime, formatter).minusMinutes(30);
            String thirtyTime = modifiedDateTime.format(formatter);
            dnbInstant.setStartTime(thirtyTime);
            dnbInstant.setEndTime(nowTime);
        }
        return recordDnbInstantMapper.selectRecordDnbInstantList(dnbInstant);
    }

    /**
     * 定时任务用：通过定时任务获取电实时数据
     * */
    //todo 通过tde 将电实时数据存入到电整点数据中
    @Override
    public void getDnbInstantByJob() {
        TdHistorySelectDto tdHistorySelectDto = new TdHistorySelectDto();
        tdHistorySelectDto.setFirstFieldName("ts");
        tdHistorySelectDto.setDatabaseName(TdEngineConstants.EMS_DATABASE_NAME);
        List<BaseEnergyType> baseEnergyTypes = baseEnergyTypeService.selectBaseEnergyTypeList(new BaseEnergyType());

        //获取当前时间与一小时前时间
       long  endTime= System.currentTimeMillis();
       //从一小时前开始，十点的话就是从九点开始，不是九点零一，开始就
       long startTime = System.currentTimeMillis()-60*60*1000-10;
        tdHistorySelectDto.setStartTime(startTime);
        tdHistorySelectDto.setEndTime(endTime);
        Date date = new Date();
        int hours = date.getHours();
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        //获取所有的计量设备
        List<BaseMonitorInfo> baseMonitorInfos = baseMonitorInfoService.selectBaseMonitorInfoList(new BaseMonitorInfo());
        List<ReportPointDnb> reportPoints = new ArrayList<ReportPointDnb>();
        //根据计量设备id获取能源表名称
        if (!baseMonitorInfos.isEmpty()){
        for (BaseMonitorInfo baseMonitorInfo : baseMonitorInfos) {
            try {
            String emsTableName = TdEngineConstants.getEmsTableName(baseMonitorInfo.getMonitorId());
            //获取当前数据以及一小时前的数据
            tdHistorySelectDto.setTableName(emsTableName);
//            tdHistorySelectDto.setTableName("t_e_ceshi7");
            List<Map<String, Object>> dnbInstant =remoteTdEngineService.getHistoryData(tdHistorySelectDto, SecurityConstants.INNER).getData().getDataList();
                R<TdReturnDataVo> historyData = remoteTdEngineService.getHistoryData(tdHistorySelectDto, SecurityConstants.INNER);
                ReportPointDnb report = new ReportPointDnb();
            report.setMonitorId(baseMonitorInfo.getMonitorId());
            report.setMonitorName(baseMonitorInfo.getMonitorName());
            report.setpMonitorId(String.valueOf(baseMonitorInfo.getParentId()));
            report.setAddress(baseMonitorInfo.getMonitorAddr());
            BigDecimal  MaxZXYG= BigDecimal.valueOf(Double.valueOf(String.valueOf(dnbInstant.get(dnbInstant.size()-1).get("zxyg"))));
            BigDecimal MinZXYG = BigDecimal.valueOf(Double.valueOf(String.valueOf(dnbInstant.get(0).get("zxyg"))));
            //pt ct值暂时无用，默认为1 需要时从计量设备信息中根据id获取
                int pt = 1;
                int ct = 1;
            //乘ct值和pt值
                /**
                 * (MAX(ifnull(t1.zxyg,0)) - MIN(ifnull(t1.zxyg,0)))  * ifnull(t2.pt,1) * ifnull(t2.ct,1)as expend,
                 * 				0 as price,
                 * 				hour(date_sub(@nowDate, interval 1 hour)) as point_time,
                 * 				DATE_FORMAT(date_sub(@nowDate, interval 1 hour),'%Y-%m-%d %H:00:00') as begin_time,
                 * 				DATE_FORMAT(@nowDate,'%Y-%m-%d %H:00:00') as end_time,
                 * 				DATE_FORMAT(date_sub(@nowDate,interval 1 hour),'%Y-%m-%d %H:00:00') as record_time*/
                report.setExpend(MaxZXYG.subtract(MinZXYG).multiply(new BigDecimal(pt)).multiply(new BigDecimal(ct)));
            report.setPrice(new BigDecimal(0));
            report.setPointTime(String.valueOf(hours-1));
            report.setMeterValue(MaxZXYG.subtract(MinZXYG));
            report.setRecordTime(sdf.parse(sdf.format(startTime)));
            report.setBeginTime(sdf.parse(sdf.format(startTime)));
            report.setEndTime(sdf.parse(sdf.format(endTime)));
                reportPoints.add(report);
            } catch (ParseException e) {
                e.printStackTrace();
            }
        }}
        //将获取到的数据存入到实时数据中
        for (ReportPointDnb p : reportPoints) {
            reportPointDnbService.insertReportPointDnb(p);
        }

    }


}
