package com.hw.wms.controller;

import java.util.List;
import java.io.IOException;
import javax.servlet.http.HttpServletResponse;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.hw.common.log.annotation.Log;
import com.hw.common.log.enums.BusinessType;
import com.hw.common.security.annotation.RequiresPermissions;
import com.hw.wms.domain.WmsInventoryCheck;
import com.hw.wms.service.IWmsInventoryCheckService;
import com.hw.common.core.web.controller.BaseController;
import com.hw.common.core.web.domain.AjaxResult;
import com.hw.common.core.utils.poi.ExcelUtil;
import com.hw.common.core.web.page.TableDataInfo;

/**
 * 盘点记录Controller
 * 
 * @author xs
 * @date 2023-12-20
 */
@RestController
@RequestMapping("/wmscheck")
public class WmsInventoryCheckController extends BaseController
{
    @Autowired
    private IWmsInventoryCheckService wmsInventoryCheckService;

    /**
     * 查询盘点记录列表
     */
    @RequiresPermissions("wms:wmscheck:list")
    @GetMapping("/list")
    public TableDataInfo list(WmsInventoryCheck wmsInventoryCheck)
    {
        startPage();
        List<WmsInventoryCheck> list = wmsInventoryCheckService.selectWmsInventoryCheckList(wmsInventoryCheck);
        return getDataTable(list);
    }

    /**
     * 导出盘点记录列表
     */
    @RequiresPermissions("wms:wmscheck:export")
    @Log(title = "盘点记录", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, WmsInventoryCheck wmsInventoryCheck)
    {
        List<WmsInventoryCheck> list = wmsInventoryCheckService.selectWmsInventoryCheckList(wmsInventoryCheck);
        ExcelUtil<WmsInventoryCheck> util = new ExcelUtil<WmsInventoryCheck>(WmsInventoryCheck.class);
        util.exportExcel(response, list, "盘点记录数据");
    }

    /**
     * 获取盘点记录详细信息
     */
    @RequiresPermissions("wms:wmscheck:query")
    @GetMapping(value = "/{inventoryCheckId}")
    public AjaxResult getInfo(@PathVariable("inventoryCheckId") Long inventoryCheckId)
    {
        return success(wmsInventoryCheckService.selectWmsInventoryCheckByInventoryCheckId(inventoryCheckId));
    }

    /**
     * 新增盘点记录
     */
    @RequiresPermissions("wms:wmscheck:add")
    @Log(title = "盘点记录", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody WmsInventoryCheck wmsInventoryCheck)
    {
        return toAjax(wmsInventoryCheckService.insertWmsInventoryCheck(wmsInventoryCheck));
    }

    /**
     * 修改盘点记录
     */
    @RequiresPermissions("wms:wmscheck:edit")
    @Log(title = "盘点记录", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody WmsInventoryCheck wmsInventoryCheck)
    {
        return toAjax(wmsInventoryCheckService.updateWmsInventoryCheck(wmsInventoryCheck));
    }

    /**
     * 删除盘点记录
     */
    @RequiresPermissions("wms:wmscheck:remove")
    @Log(title = "盘点记录", businessType = BusinessType.DELETE)
	@DeleteMapping("/{inventoryCheckIds}")
    public AjaxResult remove(@PathVariable Long[] inventoryCheckIds)
    {
        return toAjax(wmsInventoryCheckService.deleteWmsInventoryCheckByInventoryCheckIds(inventoryCheckIds));
    }
}
