package com.hw.wms.controller;

import java.util.List;
import java.io.IOException;
import javax.servlet.http.HttpServletResponse;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.hw.common.log.annotation.Log;
import com.hw.common.log.enums.BusinessType;
import com.hw.common.security.annotation.RequiresPermissions;
import com.hw.wms.domain.WmsProductStock;
import com.hw.wms.service.IWmsProductStockService;
import com.hw.common.core.web.controller.BaseController;
import com.hw.common.core.web.domain.AjaxResult;
import com.hw.common.core.utils.poi.ExcelUtil;
import com.hw.common.core.web.page.TableDataInfo;

/**
 * 成品库存Controller
 * 
 * @author xs
 * @date 2023-12-20
 */
@RestController
@RequestMapping("/productstock")
public class WmsProductStockController extends BaseController
{
    @Autowired
    private IWmsProductStockService wmsProductStockService;

    /**
     * 查询成品库存列表
     */
    @RequiresPermissions("wms:productstock:list")
    @GetMapping("/list")
    public TableDataInfo list(WmsProductStock wmsProductStock)
    {
        startPage();
        List<WmsProductStock> list = wmsProductStockService.selectWmsProductStockList(wmsProductStock);
        return getDataTable(list);
    }

    /**
     * 导出成品库存列表
     */
    @RequiresPermissions("wms:productstock:export")
    @Log(title = "成品库存", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, WmsProductStock wmsProductStock)
    {
        List<WmsProductStock> list = wmsProductStockService.selectWmsProductStockList(wmsProductStock);
        ExcelUtil<WmsProductStock> util = new ExcelUtil<WmsProductStock>(WmsProductStock.class);
        util.exportExcel(response, list, "成品库存数据");
    }

    /**
     * 获取成品库存详细信息
     */
    @RequiresPermissions("wms:productstock:query")
    @GetMapping(value = "/{productStockId}")
    public AjaxResult getInfo(@PathVariable("productStockId") Long productStockId)
    {
        return success(wmsProductStockService.selectWmsProductStockByProductStockId(productStockId));
    }

    /**
     * 新增成品库存
     */
    @RequiresPermissions("wms:productstock:add")
    @Log(title = "成品库存", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody WmsProductStock wmsProductStock)
    {
        return toAjax(wmsProductStockService.insertWmsProductStock(wmsProductStock));
    }

    /**
     * 修改成品库存
     */
    @RequiresPermissions("wms:productstock:edit")
    @Log(title = "成品库存", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody WmsProductStock wmsProductStock)
    {
        return toAjax(wmsProductStockService.updateWmsProductStock(wmsProductStock));
    }

    /**
     * 删除成品库存
     */
    @RequiresPermissions("wms:productstock:remove")
    @Log(title = "成品库存", businessType = BusinessType.DELETE)
	@DeleteMapping("/{productStockIds}")
    public AjaxResult remove(@PathVariable Long[] productStockIds)
    {
        return toAjax(wmsProductStockService.deleteWmsProductStockByProductStockIds(productStockIds));
    }
}
