package com.hw.wms.controller;

import java.util.List;
import java.io.IOException;
import javax.servlet.http.HttpServletResponse;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.hw.common.log.annotation.Log;
import com.hw.common.log.enums.BusinessType;
import com.hw.common.security.annotation.RequiresPermissions;
import com.hw.wms.domain.WmsRawStock;
import com.hw.wms.service.IWmsRawStockService;
import com.hw.common.core.web.controller.BaseController;
import com.hw.common.core.web.domain.AjaxResult;
import com.hw.common.core.utils.poi.ExcelUtil;
import com.hw.common.core.web.page.TableDataInfo;

/**
 * 原材料库存Controller
 * 
 * @author xs
 * @date 2023-12-20
 */
@RestController
@RequestMapping("/rawstock")
public class WmsRawStockController extends BaseController
{
    @Autowired
    private IWmsRawStockService wmsRawStockService;

    /**
     * 查询原材料库存列表
     */
    @RequiresPermissions("wms:rawstock:list")
    @GetMapping("/list")
    public TableDataInfo list(WmsRawStock wmsRawStock)
    {
        startPage();
        List<WmsRawStock> list = wmsRawStockService.selectWmsRawStockList(wmsRawStock);
        return getDataTable(list);
    }

    /**
     * 导出原材料库存列表
     */
    @RequiresPermissions("wms:rawstock:export")
    @Log(title = "原材料库存", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, WmsRawStock wmsRawStock)
    {
        List<WmsRawStock> list = wmsRawStockService.selectWmsRawStockList(wmsRawStock);
        ExcelUtil<WmsRawStock> util = new ExcelUtil<WmsRawStock>(WmsRawStock.class);
        util.exportExcel(response, list, "原材料库存数据");
    }

    /**
     * 获取原材料库存详细信息
     */
    @RequiresPermissions("wms:rawstock:query")
    @GetMapping(value = "/{rawStockId}")
    public AjaxResult getInfo(@PathVariable("rawStockId") Long rawStockId)
    {
        return success(wmsRawStockService.selectWmsRawStockByRawStockId(rawStockId));
    }

    /**
     * 新增原材料库存
     */
    @RequiresPermissions("wms:rawstock:add")
    @Log(title = "原材料库存", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody WmsRawStock wmsRawStock)
    {
        return toAjax(wmsRawStockService.insertWmsRawStock(wmsRawStock));
    }

    /**
     * 修改原材料库存
     */
    @RequiresPermissions("wms:rawstock:edit")
    @Log(title = "原材料库存", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody WmsRawStock wmsRawStock)
    {
        return toAjax(wmsRawStockService.updateWmsRawStock(wmsRawStock));
    }

    /**
     * 删除原材料库存
     */
    @RequiresPermissions("wms:rawstock:remove")
    @Log(title = "原材料库存", businessType = BusinessType.DELETE)
	@DeleteMapping("/{rawStockIds}")
    public AjaxResult remove(@PathVariable Long[] rawStockIds)
    {
        return toAjax(wmsRawStockService.deleteWmsRawStockByRawStockIds(rawStockIds));
    }
}
