package com.hw.wms.mapper;

import java.util.List;
import com.hw.wms.domain.WmsRawInstock;
import com.hw.wms.domain.WmsRawInstockDetail;

/**
 * 原材料入库记录Mapper接口
 *
 * @author xs
 * @date 2023-12-20
 */
public interface WmsRawInstockMapper
{
    /**
     * 查询原材料入库记录
     *
     * @param rawInstockId 原材料入库记录主键
     * @return 原材料入库记录
     */
    public WmsRawInstock selectWmsRawInstockByRawInstockId(Long rawInstockId);

    /**
     * 查询原材料入库记录列表
     *
     * @param wmsRawInstock 原材料入库记录
     * @return 原材料入库记录集合
     */
    public List<WmsRawInstock> selectWmsRawInstockList(WmsRawInstock wmsRawInstock);

    /**
     * 新增原材料入库记录
     *
     * @param wmsRawInstock 原材料入库记录
     * @return 结果
     */
    public int insertWmsRawInstock(WmsRawInstock wmsRawInstock);

    /**
     * 修改原材料入库记录
     *
     * @param wmsRawInstock 原材料入库记录
     * @return 结果
     */
    public int updateWmsRawInstock(WmsRawInstock wmsRawInstock);

    /**
     * 删除原材料入库记录
     *
     * @param rawInstockId 原材料入库记录主键
     * @return 结果
     */
    public int deleteWmsRawInstockByRawInstockId(Long rawInstockId);

    /**
     * 批量删除原材料入库记录
     *
     * @param rawInstockIds 需要删除的数据主键集合
     * @return 结果
     */
    public int deleteWmsRawInstockByRawInstockIds(Long[] rawInstockIds);

    /**
     * 批量删除原材料入库记录明细
     *
     * @param rawInstockIds 需要删除的数据主键集合
     * @return 结果
     */
    public int deleteWmsRawInstockDetailByRawInstockIds(Long[] rawInstockIds);

    /**
     * 批量新增原材料入库记录明细
     *
     * @param wmsRawInstockDetailList 原材料入库记录明细列表
     * @return 结果
     */
    public int batchWmsRawInstockDetail(List<WmsRawInstockDetail> wmsRawInstockDetailList);


    /**
     * 通过原材料入库记录主键删除原材料入库记录明细信息
     *
     * @param rawInstockId 原材料入库记录ID
     * @return 结果
     */
    public int deleteWmsRawInstockDetailByRawInstockId(Long rawInstockId);


    /**
     * 新增原材料入库记录明细
     *
     * @param wmsRawInstockDetail 原材料入库记录明细
     * @return 结果
     */
    public int insertWmsRawInstockDetail(WmsRawInstockDetail wmsRawInstockDetail);


}
