package com.hw.wms.service.impl;

import java.util.List;
import com.hw.common.core.utils.DateUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.hw.wms.mapper.WmsBaseLocationMapper;
import com.hw.wms.domain.WmsBaseLocation;
import com.hw.wms.service.IWmsBaseLocationService;

/**
 * 库位Service业务层处理
 * 
 * @author xs
 * @date 2023-12-20
 */
@Service
public class WmsBaseLocationServiceImpl implements IWmsBaseLocationService 
{
    @Autowired
    private WmsBaseLocationMapper wmsBaseLocationMapper;

    /**
     * 查询库位
     * 
     * @param locationId 库位主键
     * @return 库位
     */
    @Override
    public WmsBaseLocation selectWmsBaseLocationByLocationId(Long locationId)
    {
        return wmsBaseLocationMapper.selectWmsBaseLocationByLocationId(locationId);
    }

    /**
     * 查询库位列表
     * 
     * @param wmsBaseLocation 库位
     * @return 库位
     */
    @Override
    public List<WmsBaseLocation> selectWmsBaseLocationList(WmsBaseLocation wmsBaseLocation)
    {
        return wmsBaseLocationMapper.selectWmsBaseLocationList(wmsBaseLocation);
    }

    /**
     * 新增库位
     * 
     * @param wmsBaseLocation 库位
     * @return 结果
     */
    @Override
    public int insertWmsBaseLocation(WmsBaseLocation wmsBaseLocation)
    {
        wmsBaseLocation.setCreateTime(DateUtils.getNowDate());
        return wmsBaseLocationMapper.insertWmsBaseLocation(wmsBaseLocation);
    }

    /**
     * 修改库位
     * 
     * @param wmsBaseLocation 库位
     * @return 结果
     */
    @Override
    public int updateWmsBaseLocation(WmsBaseLocation wmsBaseLocation)
    {
        wmsBaseLocation.setUpdateTime(DateUtils.getNowDate());
        return wmsBaseLocationMapper.updateWmsBaseLocation(wmsBaseLocation);
    }

    /**
     * 批量删除库位
     * 
     * @param locationIds 需要删除的库位主键
     * @return 结果
     */
    @Override
    public int deleteWmsBaseLocationByLocationIds(Long[] locationIds)
    {
        return wmsBaseLocationMapper.deleteWmsBaseLocationByLocationIds(locationIds);
    }

    /**
     * 删除库位信息
     * 
     * @param locationId 库位主键
     * @return 结果
     */
    @Override
    public int deleteWmsBaseLocationByLocationId(Long locationId)
    {
        return wmsBaseLocationMapper.deleteWmsBaseLocationByLocationId(locationId);
    }
}
