package com.hw.wms.service.impl;

import java.util.List;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import java.util.ArrayList;
import com.hw.common.core.utils.StringUtils;
import org.springframework.transaction.annotation.Transactional;
import com.hw.wms.domain.WmsProductInstockDetail;
import com.hw.wms.mapper.WmsProductInstockMapper;
import com.hw.wms.domain.WmsProductInstock;
import com.hw.wms.service.IWmsProductInstockService;

/**
 * 成品入库记录Service业务层处理
 * 
 * @author xs
 * @date 2023-12-20
 */
@Service
public class WmsProductInstockServiceImpl implements IWmsProductInstockService 
{
    @Autowired
    private WmsProductInstockMapper wmsProductInstockMapper;

    /**
     * 查询成品入库记录
     * 
     * @param productInstockId 成品入库记录主键
     * @return 成品入库记录
     */
    @Override
    public WmsProductInstock selectWmsProductInstockByProductInstockId(Long productInstockId)
    {
        return wmsProductInstockMapper.selectWmsProductInstockByProductInstockId(productInstockId);
    }

    /**
     * 查询成品入库记录列表
     * 
     * @param wmsProductInstock 成品入库记录
     * @return 成品入库记录
     */
    @Override
    public List<WmsProductInstock> selectWmsProductInstockList(WmsProductInstock wmsProductInstock)
    {
        return wmsProductInstockMapper.selectWmsProductInstockList(wmsProductInstock);
    }

    /**
     * 新增成品入库记录
     * 
     * @param wmsProductInstock 成品入库记录
     * @return 结果
     */
    @Transactional
    @Override
    public int insertWmsProductInstock(WmsProductInstock wmsProductInstock)
    {
        int rows = wmsProductInstockMapper.insertWmsProductInstock(wmsProductInstock);
        insertWmsProductInstockDetail(wmsProductInstock);
        return rows;
    }

    /**
     * 修改成品入库记录
     * 
     * @param wmsProductInstock 成品入库记录
     * @return 结果
     */
    @Transactional
    @Override
    public int updateWmsProductInstock(WmsProductInstock wmsProductInstock)
    {
        wmsProductInstockMapper.deleteWmsProductInstockDetailByProductInstockId(wmsProductInstock.getProductInstockId());
        insertWmsProductInstockDetail(wmsProductInstock);
        return wmsProductInstockMapper.updateWmsProductInstock(wmsProductInstock);
    }

    /**
     * 批量删除成品入库记录
     * 
     * @param productInstockIds 需要删除的成品入库记录主键
     * @return 结果
     */
    @Transactional
    @Override
    public int deleteWmsProductInstockByProductInstockIds(Long[] productInstockIds)
    {
        wmsProductInstockMapper.deleteWmsProductInstockDetailByProductInstockIds(productInstockIds);
        return wmsProductInstockMapper.deleteWmsProductInstockByProductInstockIds(productInstockIds);
    }

    /**
     * 删除成品入库记录信息
     * 
     * @param productInstockId 成品入库记录主键
     * @return 结果
     */
    @Transactional
    @Override
    public int deleteWmsProductInstockByProductInstockId(Long productInstockId)
    {
        wmsProductInstockMapper.deleteWmsProductInstockDetailByProductInstockId(productInstockId);
        return wmsProductInstockMapper.deleteWmsProductInstockByProductInstockId(productInstockId);
    }

    /**
     * 新增成品入库记录明细信息
     * 
     * @param wmsProductInstock 成品入库记录对象
     */
    public void insertWmsProductInstockDetail(WmsProductInstock wmsProductInstock)
    {
        List<WmsProductInstockDetail> wmsProductInstockDetailList = wmsProductInstock.getWmsProductInstockDetailList();
        Long productInstockId = wmsProductInstock.getProductInstockId();
        if (StringUtils.isNotNull(wmsProductInstockDetailList))
        {
            List<WmsProductInstockDetail> list = new ArrayList<WmsProductInstockDetail>();
            for (WmsProductInstockDetail wmsProductInstockDetail : wmsProductInstockDetailList)
            {
                wmsProductInstockDetail.setProductInstockId(productInstockId);
                list.add(wmsProductInstockDetail);
            }
            if (list.size() > 0)
            {
                wmsProductInstockMapper.batchWmsProductInstockDetail(list);
            }
        }
    }
}
